@@ -16,6 +16,7 @@ import time
 from . import pipeline
 from azure.iot.device.common.auth import connection_string as cs
 from azure.iot.device.common.auth import sastoken as st
+from azure.iot.device.iothub import client_event
 from azure.iot.device import exceptions
 from azure.iot.device.common import auth
 from . import edge_hsm
@@ -100,7 +101,7 @@ RECEIVE_TYPE_API = "api"  # Only use APIs for receive
 
 @six.add_metaclass(abc.ABCMeta)
 class AbstractIoTHubClient(object):
-    """ A superclass representing a generic IoTHub client.
+    """A superclass representing a generic IoTHub client.
     This class needs to be extended for specific clients.
     """
 
@@ -113,25 +114,37 @@ class AbstractIoTHubClient(object):
         self._mqtt_pipeline = mqtt_pipeline
         self._http_pipeline = http_pipeline
 
-        self._inbox_manager = None  # this will be overriden in child class
-        self._handler_manager = None  # this will be overriden in child class
+        self._inbox_manager = None  # this will be overridden in child class
+        self._handler_manager = None  # this will be overridden in child class
         self._receive_type = RECEIVE_TYPE_NONE_SET
         self._client_lock = threading.Lock()
 
     def _on_connected(self):
         """Helper handler that is called upon an iothub pipeline connect"""
         logger.info("Connection State - Connected")
+        client_event_inbox = self._inbox_manager.get_client_event_inbox()
+        event = client_event.ClientEvent(client_event.CONNECTION_STATE_CHANGE)
+        client_event_inbox.put(event)
         # Ensure that all handlers are running now that connection is re-established.
         self._handler_manager.ensure_running()
 
     def _on_disconnected(self):
         """Helper handler that is called upon an iothub pipeline disconnect"""
         logger.info("Connection State - Disconnected")
+        client_event_inbox = self._inbox_manager.get_client_event_inbox()
+        event = client_event.ClientEvent(client_event.CONNECTION_STATE_CHANGE)
+        client_event_inbox.put(event)
         # Locally stored method requests on client are cleared.
         # They will be resent by IoTHub on reconnect.
         self._inbox_manager.clear_all_method_requests()
         logger.info("Cleared all pending method requests due to disconnect")
 
+    # def _on_new_sastoken_required(self):
+    #     logger.info("New SasToken required from user")
+    #     client_event_inbox = self._inbox_manager.get_client_event_inbox()
+    #     event = client_event.ClientEvent(client_event.NEW_SASTOKEN_REQUIRED)
+    #     client_event_inbox.put(event)
+
     def _check_receive_mode_is_api(self):
         """Call this function first in EVERY receive API"""
         with self._client_lock:
@@ -403,6 +416,30 @@ class AbstractIoTHubClient(object):
     def on_twin_desired_properties_patch_received(self):
         pass
 
+    @property
+    def on_connection_state_change(self):
+        return self._handler_manager.on_connection_state_change
+
+    @on_connection_state_change.setter
+    def on_connection_state_change(self, value):
+        self._handler_manager.on_connection_state_change = value
+
+    # @property
+    # def on_new_sastoken_required(self):
+    #     return self._handler_manager.on_new_sastoken_required
+
+    # @on_new_sastoken_required.setter
+    # def on_new_sastoken_required(self, value):
+    #     self._handler_manager.on_new_sastoken_required = value
+
+    # @property
+    # def on_background_exception(self):
+    #     return self._handler_manager.on_background_exception
+
+    # @on_background_exception.setter
+    # def on_background_exception(self, value):
+    #     self._handler_manager.on_background_exception = value
+
 
 @six.add_metaclass(abc.ABCMeta)
 class AbstractIoTHubDeviceClient(AbstractIoTHubClient):
