@@ -8,6 +8,7 @@ import logging
 import sys
 from azure.iot.device.common.evented_callback import EventedCallback
 from azure.iot.device.common.pipeline import (
+    pipeline_events_base,
     pipeline_stages_base,
     pipeline_ops_base,
     pipeline_stages_mqtt,
@@ -40,9 +41,11 @@ class MQTTPipeline(object):
             constant.TWIN_PATCHES: False,
         }
 
-        # Event Handlers - Will be set by Client after instantiation of this object
+        # Handlers - Will be set by Client after instantiation of this object
         self.on_connected = None
         self.on_disconnected = None
+        # self.on_new_sastoken_required = None
+
         self.on_c2d_message_received = None
         self.on_input_message_received = None
         self.on_method_request_received = None
@@ -145,6 +148,12 @@ class MQTTPipeline(object):
                 else:
                     logger.error("Twin patch event received with no handler. Dropping.")
 
+            # elif isinstance(event, pipeline_events_base.NewSasTokenRequired):
+            #     if self.on_new_sastoken_required:
+            #         self.on_new_sastoken_required()
+            #     else:
+            #         logger.error("New sastoken required event received with no handler. Dropping.")
+
             else:
                 logger.error("Dropping unknown pipeline event {}".format(event.name))
 
